/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 */

package com.nemmco.participantbatcher;

import java.util.Calendar;

import java.io.File;

import java.text.SimpleDateFormat;

import org.apache.log4j.Logger;

import com.nemmco.participantbatcher.process.BusinessProcess;
import com.nemmco.participantbatcher.process.ResendProcess;
import com.nemmco.participantbatcher.process.CleanupProcess;
import com.nemmco.participantbatcher.process.FileManager;
import com.nemmco.participantbatcher.process.BatcherFileFilter;

import com.nemmco.participantbatcher.util.Constants;
import com.nemmco.participantbatcher.util.HttpShutdownHook;
import com.nemmco.participantbatcher.util.HttpServer;
import com.nemmco.participantbatcher.util.Util_Logger;
import com.nemmco.participantbatcher.util.MonitorLogger;
import com.nemmco.participantbatcher.util.ApplicationException;
import com.nemmco.participantbatcher.util.PropertiesReader;

/**
 * This is the main class of the Participant Batcher application.
 * It starts all process threads. It will also be used as the
 * monitoring thread.
 *
 * @author Kevin Jin
 * version 1.0 - 28 Aug 2003 - Creation
 * version 1.1 - 4 Nov 2003 modified logging, added version number, added loggerlevel
 * version 1.3 - 19-May-2004 added stop file processing, log4j
 * version 1.3.1 - 16-Nov-2004 fixed bug with the production of -ve ack files
 * version 1.4.0 - 28-Feb-2005 Added share as alternative to FTP
 * version 1.4.1 - 09-Sep-2005 Modified the ack to make the ID used unique
 * version 1.4.2 - 17-Dec-2005 Not get caught by the filename.ack.tmp problem this is not an .ack file
 *                  -> added custom clean function;
 *                  -> added HttpServer function
 *                  -> free/total memory added to debug monitor message
 * version 1.4.3 - 22-Mar-2006 bug fixes of v1.4.2 - defects 3771,3774,3776
 * version 1.4.4 - 22-Apr-2006 bug fixes of v1.4.3 - SendProcess,FTPEDTProcesser,logging,javaDoc,coding standards
 * version 1.5.0 - 07-Aug-2006 added webDAV support and Instance Batcher mode of opertaion
 * version 1.5.1 - 1-Nov-2006 Made BatcherFileFilter generic to handle a file type or a set of file types, replaces AckFileFilter
 * version 1.5.2 - 13-Apr-2007 Fix sending of B2B to send any file that is schema invalid or zip invalid (so that it gets -ve acked), fixSpecialCodes as well
 * version 1.5.3 - 30-Jun-2007 Fix the stop file confusion between INSTANCE and INSTANCEEXTRA by including "." in file mask
 * version 1.5.4 - 15-Feb-2008 Emergency HSM fix in java 1.6.0
 * version 2.0.0 - 15-Sep-2010 6356 Update version number to V2.0 for Java 1.6 upgrade.
 * version 2.0.1 - 11-Nov-2010 Fix the problem with renaming .sent files back to .zip when hub has acked
 * version 2.0.2 - 19-Nov-2010 Peer review work
 * @version 2.0.3 - 22-Feb-2011 Ensure process stop if critical thread fails
 */

public class ParticipantBatcher {

   private static Logger logger = Logger.getLogger(ParticipantBatcher.class);

   /**
    * Constructor
    */
   public ParticipantBatcher() {}

   /** The main method of the Participant Batcher application.
    * This is the entry point of this application.
    *
    * @param args instance_name String[]
    */
   public static void main(String[] args) {

      ParticipantBatcher pb = new ParticipantBatcher();
      String versionNumber = Constants.APPLICATION_VERSION;

      /* First, we have to retrieve all arguments
         * Note that there should be only 1 argument - the instance name
         */
      String instanceName = "";
      if (args.length != 1) {
         System.out.println("Usage: java -classpath \"......\" " + "com.nemmco.participantbatcher.ParticipantBatcher " + "instanceName");
         System.exit(1);
      } else {
         instanceName = args[0];
      }

      try {

         // Now let's load properties from the instance's properties file
         pb.loadProperties(instanceName);

         // put list of constants from properties onto log
         Constants.getConstant().debugConstants();

         // Start the shutdown hook
         Runtime rt = Runtime.getRuntime();
         HttpShutdownHook hsh = new HttpShutdownHook();
         rt.addShutdownHook(hsh);

         // Start the main business processes
         BusinessProcess bp = new BusinessProcess();
         bp.start();

         // Start the resend business processes
         ResendProcess rp = new ResendProcess();
         if (!Constants.PROCESS_RECEIVE.equalsIgnoreCase(Constants.getConstant().getProcessType())) {
            rp.start();
         }

         // Start the cleanup process
         CleanupProcess cp = new CleanupProcess();
         cp.start();

         // Start web interface - only starts if port has been set to > 0
         HttpServer hs = new HttpServer();
         hs.start();

         // tell the user that startup has finished
         SimpleDateFormat dateFormat;
         dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

         System.out.println("" + dateFormat.format((Calendar.getInstance()).getTime()) + " Instance " + instanceName + " has started, version " + versionNumber);
         Util_Logger.info(logger, "Instance " + instanceName + " has started, version " + versionNumber);

         /* The main thread of this application will
             * also be used as the monitoring thread
             */
         //Thread t = new Thread();
         String monitorFileName = Constants.getConstant().getMonitorFileDir() + "/" + instanceName + "_monitor.log";
         File monitorFile = new File(monitorFileName);
         while (true) {
            Thread.sleep(Constants.getConstant().getMonitorWaitTime());

            // test threads, if they are all alive, write a message to the monitor log
            boolean isAllAlive = false;

            if (!Constants.PROCESS_RECEIVE.equalsIgnoreCase(Constants.getConstant().getProcessType())) {
               if (cp.isAlive() && bp.isAlive() && rp.isAlive()) {
                  isAllAlive = true;
               }
            } else {
               if (cp.isAlive() && bp.isAlive()) {
                  isAllAlive = true;
               }
            }

            // update the monitor file only if all threads ok and main process is connected
            if (isAllAlive && bp.getConnected()) {
               Util_Logger.debug(logger,
                                 "Monitor thread -- all is ok. Free mem " + rt.freeMemory() / 1024 / 1024 + " Max mem " + rt.maxMemory() / 1024 / 1024 + " MB");
               MonitorLogger.log(monitorFile);
            } else if (isAllAlive) {
               Util_Logger.warning(logger, "Monitor thread -- no communication connection");
            } else {
               // this case one or more thread(s) failed
               Util_Logger.error(logger, "Monitor thread -- one or more thread(s) are not alive - restart");
            }

            // check for stop file instanceName.stp in stopFilePath v1.5.3
            FileManager fm = new FileManager();
            String[] stopFileList = fm.scanDir(Constants.getConstant().getStopFileDir(), new BatcherFileFilter(".stp", instanceName + "."));

            // stop if file exists - this is a clean way to stop it
            if ((stopFileList != null) && (stopFileList.length > 0)) {
               // keep users informed and stop
               Util_Logger.warning(logger, "Stopping the processes");
               System.out.println("" + dateFormat.format((Calendar.getInstance()).getTime()) + " Stop File Exists " + stopFileList[0] + " Stopping instance " + instanceName);

               // stop the threads
               if (!Constants.PROCESS_RECEIVE.equalsIgnoreCase(Constants.getConstant().getProcessType())) {
                  cp.requestStop();
                  bp.requestStop();
                  rp.requestStop();
                  hs.requestStop();
               } else {
                  cp.requestStop();
                  bp.requestStop();
                  hs.requestStop();
               }

               boolean isAnyAlive = false;
               int waitTime = 0;

               /* check if the threads have stopped
                       * loop if they havent stopped up to a wait time
                       */
               do {
                  isAnyAlive = false;
                  if (!Constants.PROCESS_RECEIVE.equalsIgnoreCase(Constants.getConstant().getProcessType())) {
                     if (cp.isAlive() || bp.isAlive() || rp.isAlive()) {
                        isAnyAlive = true;
                     }
                  } else {
                     if (cp.isAlive() || bp.isAlive()) {
                        isAnyAlive = true;
                     }
                  }

                  Thread.sleep(1000);
                  waitTime = waitTime + 1000;

               } while (isAnyAlive && (Constants.getConstant().getStopFileWaitTime() > waitTime));

               // keep users informed and stop
               Util_Logger.warning(logger, "Stopped all processes");
               System.out.println("" + dateFormat.format((Calendar.getInstance()).getTime()) + " Stopped instance ");

               // delete the stop file (otherwise app stops immediately next start)
               fm.deleteFile(Constants.getConstant().getStopFileDir() + "/" + instanceName + ".stp");
               System.exit(0);

            } else {
               //System.out.println(" Continues ");
            }
         }
      } catch (Exception e) {
         if (e instanceof ApplicationException) {

            // Fatal error occured during the application running
            ApplicationException ae = (ApplicationException)e;
            System.out.println("Application error happened when running this program, Please check the log file");
            if (!ae.getLogID().isEmpty()) {
               System.out.println("The error ID is: " + ae.getLogID());
            }
            System.out.println("The error is: " + ae.getErrorMessage());
            System.exit(1);
         } else {

            // Can not start the application
            System.out.println("Error happened when starting this application, Please try again.");
            System.out.println("If problem persist, Please inform technical support, and quote below.");
            System.out.println("Exception was " + e.getMessage());
            e.printStackTrace();
         }
      }
   }

   // Load properties
   private void loadProperties(String instanceName) throws Exception {
      //PropertiesReader pr = new PropertiesReader();  // java 1.6
      PropertiesReader.readProperties(instanceName);
   }
}

