/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 * 28-Aug-2012     Oleg Pevzner       7984 Java upgrade from 6 to 7
 */

package com.nemmco.participantbatcher.process;

import com.nemmco.participantbatcher.util.Constants;
import com.nemmco.participantbatcher.util.Util_Logger;

import java.io.File;
import java.io.FilenameFilter;

import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

/**
 * This class is used to ensure that only specific files are manupilated.
 * <p>
 * @author Kevin Jin
 * version 1.0 - 04 Sep 2003 - Creation
 * version 1.3 - 20 Aug 2004 - Modified to look at the extension, not string at any place in the filename, log4j
 * version 1.4.2. - 4 Feb 2006 - Added toString
 * version 1.4.3 - 22-Mar-2006 - Modified toString
 * version 1.4.4 6-May-2006 Fixed javaDoc and coding standards
 * version 1.5.1 - 1-Nov-2006 Made BatcherFileFilter generic to handle a file type or a set of file types, replaces AckFileFilter
 *                             and changes CustomCleanFilter
 * version 2.0.0 - 10-May-2010 Bob Jones Replace Jakarta Regular Expression with Regex in JDK.
 * @version 2.0.2 - 19-Nov-2010 Peer review work
 */

public class BatcherFileFilter implements FilenameFilter {

   private String[] fileExtensions;
   private Vector<String> fileTypes;
   private static Logger logger = Logger.getLogger(BatcherFileFilter.class);

   /**
    * Constructor
    * This constructor takes one paramter the file extension string such as .zip .ack
    * This can ensure that only one file type being processed at one time.
    *
    */
   public BatcherFileFilter() {
      fileExtensions = null;
   }

   /**
    * Constructor
    * This constructor takes one paramter the file extension string such as .zip .ack
    * This can ensure that only one file type being processed at one time.
    *
    * @param fileExtensionsParam String
    */
   public BatcherFileFilter(String fileExtensionsParam) {
      this();
      fileExtensions = new String[1];
      fileExtensions[0] = fileExtensionsParam;
      fileTypes = Constants.getConstant().getFileTypes();
   }

   /**
    * Constructor
    * This constructor takes one paramter the file extension string such as .stp
    * and one file mask (regular expression) as a string
    * This is used to find a particular file eg instance.stp
    *
    * @param fileExtensionsParam String
    * @param fileName String
    */
   public BatcherFileFilter(String fileExtensionsParam, String fileName) {
      this();
      fileExtensions = new String[1];
      fileExtensions[0] = fileExtensionsParam;
      fileTypes = new Vector<String>();
      fileTypes.add(fileName);
   }

   /**
    * Constructor
    * This constructor takes many file extensions such as .zip .ack
    * This can respond with a broader answer
    *
    * @param fileExtensionsParam String[]
    */
   public BatcherFileFilter(String[] fileExtensionsParam) {
      this();
      fileExtensions = fileExtensionsParam.clone();
      fileTypes = Constants.getConstant().getFileTypes();
   }

   /**
    * Constructor
    * This constructor takes many file extensions such as .zip .ack
    * This can respond with a broader answer
    *
    * @param fileExtensionsParam String[]
    */
   public BatcherFileFilter(String fileExtensionsParam, Vector<String> fileTypesParam) {
      this();
      fileExtensions = new String[1];
      fileExtensions[0] = fileExtensionsParam;
      fileTypes = fileTypesParam;
   }

   /**
    * This is the main method defined in the FilenameFilter interface
    */
   public boolean accept(File dir, String name) {

      // assume false
      boolean retStatus = false;

      try {

         // now we test if the file name extension is in the list
         for (int c0 = 0; c0 < fileExtensions.length; c0++) {
            if (name.endsWith(fileExtensions[c0]) && fileExtensions[c0].length() > 0) { // v1.5.1
               retStatus = true;
               break;
            }
         }

         // not on the list of extensions, so return false
         if (!retStatus) {
            return retStatus;
         }

         // assume false
         retStatus = false;

         // Now, we test if the file meets user specified regular expression criteria
         if (fileTypes.size() > 0) {
            for (int i = 0; i < fileTypes.size(); i++) {

               String fileType = fileTypes.elementAt(i);
               Pattern pattern = Pattern.compile(fileType);
               Matcher matcher = pattern.matcher(name);
               if (matcher.find()) {
                  retStatus = true;
                  break;
               }
            }
         } else {
            // No file types specified
            retStatus = true;
         }

      } catch (Exception e) {
         Util_Logger.warning(logger, "Error occured when using BatcherFileFilter", e);
      }
      return retStatus;
   }

   public String toString() {
      return "FileFilter, Extensions=" + this.getfileExtensions() + " Types=" + this.getFileTypes();
   }

   private String getfileExtensions() {
      StringBuffer buffer = new StringBuffer();
      buffer.append("{"); // v1.5.1
      for (int c0 = 0; c0 < fileExtensions.length; c0++) {
         buffer.append(fileExtensions[c0] + ",");
      }
      String fileExtension = buffer.toString();
      if (fileExtension.endsWith(",")) {
         fileExtension = fileExtension.substring(0, fileExtension.length() - 1);
      }
      fileExtension = fileExtension + "}";
      return fileExtension;
   }

   private String getFileTypes() {
      StringBuffer buffer = new StringBuffer();
      for (int i = 0; i < fileTypes.size(); i++) {
         buffer.append(fileTypes.elementAt(i) + ";");
      }
      return buffer.toString();
   }

}
