/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 */

package com.nemmco.participantbatcher.process;

import com.nemmco.participantbatcher.ftp.ComException;
import com.nemmco.participantbatcher.ftp.ComProcess;
import com.nemmco.participantbatcher.ftp.ComProcessManager;
import com.nemmco.participantbatcher.util.Constants;
import com.nemmco.participantbatcher.util.HttpDataStore;
import com.nemmco.participantbatcher.util.Util_Logger;

import java.text.SimpleDateFormat;

import java.util.Calendar;

import org.apache.log4j.Logger;
//import java.lang.Integer;

/**
 * This class is used to initiate the two main business processes
 * Sending and Receiving.  It makes connection via teh ComProcessManager,
 * deals with Com exceptions, logs to console the connected state, and decides
 * the send recieve order of work.
 *
 * @author Kevin Jin
 * version 1.0 - 28 Aug 2003 - Creation
 * version 1.1 - 4 Nov 2003 modified logging, added exceptwaittime
 * version 1.3 - 19-May-2004 added stop file processing, log4j, FTP exception logging
 * version 1.4.0 - 28-Feb-2005 Added share as alternative to FTP
 * version 1.4.2 - 17-Dec-2005 added check for null return fron Com processer and warning
 *                - Added HttpDataStore data
 * version 1.4.4 - 24-Apr-2006 Added a disconnect if COM failure has occured, monitor of connect state
 * version 1.5.0 - 23-June-2006 Changed stopping for Instance Batcher functionality
 * version 1.5.1 - 5-Nov-2006 added partSleep to improve reponsiveness when stopping, added time debug message
 * version 1.5.4 - 8-Aug-2007 InstanceBatcher2 return work done
 * version 2.0.1 - 15-Nov-2010 Peer review work
 * @version 2.0.2 - 19-Nov-2010 Peer review work
 */

public class BusinessProcess extends PbThread {

   // private volatile boolean stop = false;
   private volatile boolean connected = false;
   //private ReceiveProcess rp = null;  // java 1.6
   //private SendProcess sp = null; // java 1.6
   private static Logger logger = Logger.getLogger(BusinessProcess.class);
   private HttpDataStore hds;
   private boolean order = true;
   private int workCounter = 0;

   /**
    * Constructor
    */
   public BusinessProcess() {
      hds = HttpDataStore.getHttpDataStore();
      workCounter = 0;
   }

   /**
    * Thread run method
    */
   public void run() {

      while (!getStop()) {

         long waitingTime = Constants.getConstant().getProcessWaitTime();

         ComProcess processer = null;
         try {
            Util_Logger.debug(logger, "Main Business Process Started");

            processer = ComProcessManager.getInstance().getMainProcess();

            long startTime = System.currentTimeMillis();
            String[] pniFileList = processer.listNames(Constants.getConstant().getPNInboxDir());
            String[] pnoFileList = processer.listNames(Constants.getConstant().getPNOutboxDir());
            long timeTaken = System.currentTimeMillis() - startTime;
            Util_Logger.debug(logger, "Main Business Process directory scan, " + timeTaken + " msec");

            hds.setPniFileList(pniFileList);
            hds.setPnoFileList(pnoFileList);

            if (pniFileList != null && pnoFileList != null) {

               // Start the business processes
               setConnected(true);
               if (Constants.PROCESS_SEND.equalsIgnoreCase(Constants.getConstant().getProcessType())) {
                  SendProcess sp = new SendProcess();
                  workCounter = workCounter + sp.process(processer, pniFileList, pnoFileList);
               } else if (Constants.PROCESS_RECEIVE.equalsIgnoreCase(Constants.getConstant().getProcessType())) {
                  ReceiveProcess rp = new ReceiveProcess();
                  workCounter = workCounter + rp.process(processer, pniFileList, pnoFileList);
               } else if (order) { // should be Constants.PROCESS_BOTH - use order to swap the
                  // send/recieve vs recieve/send
                  order = !order;
                  ReceiveProcess rp = new ReceiveProcess();
                  workCounter = workCounter + rp.process(processer, pniFileList, pnoFileList);
                  SendProcess sp = new SendProcess();
                  workCounter = workCounter + sp.process(processer, pniFileList, pnoFileList);
               } else {
                  order = !order;
                  SendProcess sp = new SendProcess();
                  workCounter = workCounter + sp.process(processer, pniFileList, pnoFileList);
                  ReceiveProcess rp = new ReceiveProcess();
                  workCounter = workCounter + rp.process(processer, pniFileList, pnoFileList);
               }

               Util_Logger.debug(logger, "Main Business Process Finished");
               if (getStop())
                  break;

            } else {

               Util_Logger.warning(logger, "Main Business Process failed, null return from comunication processor");

               // update the http data store as Send/Recv will not
               setConnected(false);
               hds.setZipsSent(0);
               hds.setAcksRecieved(0);
               hds.setZipsRecieved(0);
               hds.setAcksSent(0);
            }

         } catch (Exception e) {

            // update HTTP data store as Send/Recieve wont
            setConnected(false);
            hds.setZipsSent(0);
            hds.setAcksRecieved(0);
            hds.setZipsRecieved(0);
            hds.setAcksSent(0);

            // there are two possibilities:
            // ComException -- Communication process failure
            if (e instanceof ComException) {

               // free up the Com process (close the control port for FTP)
               ComProcessManager.getInstance().setMainProcessToNULL();

               // deal with exception to reveal lower level exception information
               ComException ftpe = (ComException)e;
               Util_Logger.warning(logger, "Communication error occured when running main business process");
               Util_Logger.warning(logger, ftpe.getErrorMessage() + " e: " + ftpe.getOriginalComException());

               // ApplicationException -- Local file process failure
            } else {
               Util_Logger.warning(logger, "Error occured when running main business process", e);
            }

            // we need to set the waiting time to the value for the exception
            // so that we dont flood the log and overload the communication path
            waitingTime = Constants.getConstant().getExceptWaitTime();

         }

         // wait some time to control load on communication channel
         partSleep(waitingTime);

         // stop overflow
         if (workCounter > Integer.MAX_VALUE - 2) {
            workCounter = 0;
         }

      }

   }

   /** are we connected to server?
    * @return connected state as boolean
    */
   public boolean getConnected() {
      return connected;
   }

   /**
    * Used to log on the console when connected state changes as
    * well as updating HttpDataStore and a local connected flag
    * @param paramConnected boolean is true if communication is connected
    */
   private void setConnected(boolean paramConnected) {

      // have we changed state?
      if (paramConnected != connected) {

         // log state
         SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
         if (paramConnected) {
            System.out.println("" + dateFormat.format((Calendar.getInstance()).getTime()) + " Instance " + Constants.getConstant().getInstanceName() + " Connected");
         } else {
            System.out.println("" + dateFormat.format((Calendar.getInstance()).getTime()) + " Instance " + Constants.getConstant().getInstanceName() + " Disconnected");
         }
      }

      // store state for getter and in HttpDataStore
      connected = paramConnected;
      hds.setConnected(paramConnected);
   }

   public int getWorkCounter() {
      return workCounter;
   }

}
