/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 * 28-Aug-2012     Oleg Pevzner       7984 Java upgrade from 6 to 7
 * 10-Feb-2015     Warwick Mills      5252 Metering Batcher fails when duplicate file exists
 */

package com.nemmco.participantbatcher.process;

import com.nemmco.participantbatcher.util.Constants;
import com.nemmco.participantbatcher.util.Util_Logger;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;

import java.util.ArrayList;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

/**
 * This classes is used to manupulate file(s) in local directories.
 *
 * @author Kevin Jin
 *  version 1.0   -  5 Sep 2003 Creation
 *  version 1.3   - 20-Aug-2004 added fileNameCompare
 *  version 1.4.2 - 17-Dec-2005 made scanFTPFileList check for a true extension rather than just a scan for the .ext string
 *                              fixed fileNameCompare
 *  version 1.4.4 -  6-May-2006 Fixed javaDoc and coding standards
 *  version 1.5.1 -  1-Nov-2006 Made scanFTPFileList use regular expression - see BatcherFileFilter v1.5.1
 *  version 1.5.4 - 17-Aug-2007 Clarified some of the logging and added new rename for ShareProcessor
 *  version 2.0.0 - 10-May-2010 Bob Jones Replace Jakarta Regular Expression with Regex in JDK.
 *  version 2.0.2 - 19-Nov-2010 Peer review work
 * @version 2.0.7 - 10-Feb-2015 Overwrite duplicate files
 */

public class FileManager {

    private static Logger logger = Logger.getLogger(FileManager.class);

    /**
     * Constructor
     */
    public FileManager() {
    }

    /**
     * This method is used to rename a file in the same directory.
     *
     * @param dir - the file directory String
     * @param oldFileName - the existing file name String
     * @param newFileName - the new file name  String
     * @return boolean true - if file rename succeedss
     */
    public boolean renameFile(String dir, String oldFileName, String newFileName) {

        String oldFileFullName = dir + "/" + oldFileName;
        String newFileFullName = dir + "/" + newFileName;

        return renameFile(oldFileFullName, newFileFullName);

    }

    /**
     * This method is used to rename a file in the different directory.
     *
     * This is actually an action of copying a file to a defferent directory
     * using different name
     *
     * @param oldFileFullName - the existing file name String
     * @param newFileFullName - the new file name String
     * @return boolean true - if file rename succeedss
     */
    public boolean renameFile(String oldFileFullName, String newFileFullName) {

        boolean result = false;

        try {
            File oldFile = new File(oldFileFullName);
            File newFile = new File(newFileFullName);
            if (oldFile.exists()) {
                if (!newFile.getParentFile().exists()) {
                    newFile.getParentFile().mkdirs();
                }
                if (Constants.getConstant().getFileOverwrite() && newFile.exists()) {
                    if (newFile.delete()) {
                        Util_Logger.info(logger, "Overwriting existing file - " + newFileFullName);
                    } else {
                        Util_Logger.warning(logger, "Can not overwrite file - " + newFileFullName);
                    }
                }
                if (oldFile.renameTo(newFile)) {
                    // successful renamed
                    result = true;
                } else {
                    // can not rename, log the event
                    String message = "Can not rename file from " + oldFileFullName + " to " + newFileFullName + " (exists)";
                    Util_Logger.warning(logger, message);
                }
            } else {
                // file not exist
                String message = "Can not rename file from " + oldFileFullName + " to " + newFileFullName +
                    " (source file missing) " + oldFileFullName + " does not exist.";
                Util_Logger.warning(logger, message);
            }

        } catch (Exception e) {
            String message = "Can not rename file from " + oldFileFullName + " to " + newFileFullName + "(exception). " + e.getMessage();
            Util_Logger.error(logger, message, e);
        }

        return result;

    }

    public boolean renameFile(String oldDir, String oldName, String newDir, String newName) {

        String oldFileFullName = oldDir + "/" + oldName;
        String newFileFullName = newDir + "/" + newName;

        return renameFile(oldFileFullName, newFileFullName);

    }

    /**
     * This method is used to delete a file.
     *
     * @param fileName String
     * @return boolean true - if file deletion success
     */
    public boolean deleteFile(String fileName) {

        boolean result = false;

        try {
            File file = new File(fileName);

            if (file.exists()) {
                if (file.delete()) {
                    // successful deleted
                    result = true;
                } else {
                    // can not delete, log the event
                    Util_Logger.warning(logger, "Can not delete file - " + fileName);
                }
            } else {
                // file not exist
                Util_Logger.warning(logger, "Can not delete file - " + fileName + ", it does not exist");
            }

        } catch (Exception e) {
            Util_Logger.error(logger, "Can not delete file - " + fileName + ", exception occured", e);
        }

        return result;

    }

    /**
     * This method is used to copy a single file from one local directory to
     * another local directory
     *
     * @param fromDirName String
     * @param toDirName String
     * @param fileName String
     * @return boolean true - if the copy process succeed
     */
    public boolean copyLocalFile(String fromDirName, String toDirName, String fileName) {

        boolean result = false;
        String message;

        try {

            File fromDir = new File(fromDirName);
            File toDir = new File(toDirName);

            // First, we need to test if the "fromDir" and "toDir" are valid
            if (!fromDir.exists()) {
                message = "Can not copy file " + fileName + " from " + fromDirName + " to " + toDirName + ", " + fromDirName + " is not a valid directory";
                Util_Logger.warning(logger, message);
                return result;
            }

            if (!fromDir.isDirectory()) {
                message = "Can not copy file " + fileName + " from " + fromDirName + " to " + toDirName + ", " + fromDirName + " is not a valid directory";
                Util_Logger.warning(logger, message);
                return result;
            }

           if (!toDir.exists()) {
               toDir.mkdirs();
           }
            if (!toDir.exists()) {
                message = "Can not copy file " + fileName + " from " + fromDirName + " to " + toDirName + ", " + toDirName + " is not a valid directory";
                Util_Logger.warning(logger, message);
                return result;
            }

            if (!toDir.isDirectory()) {
                message = "Can not copy file " + fileName + " from " + fromDirName + " to " + toDirName + ", " + toDirName + " is not a valid directory";
                Util_Logger.warning(logger, message);
                return result;
            }

            // Now, let's copy the file
            String srcFileName = fromDirName + "/" + fileName;
            File srcFile = new File(srcFileName);

            String detFileName = toDirName + "/" + fileName;
            File detFile = new File(detFileName);

            if (srcFile.exists()) {
                if (Constants.getConstant().getFileOverwrite() && detFile.exists()) {
                    if (detFile.delete()) {
                        Util_Logger.info(logger, "Overwriting existing file - " + detFile.getAbsolutePath());
                    } else {
                        Util_Logger.warning(logger, "Can not overwrite file - " + detFile.getAbsolutePath());
                    }
                }
                if (srcFile.renameTo(detFile)) {
                    result = true;
                } else {
                    throw new Exception("Rename of file failed in copy routine");
                }
            } else {
                // file not exist
                throw new Exception("Rename of file failed in copy routine. " + srcFile.getAbsolutePath() + " does not exist.");
            }
        } catch (Exception e) {
            message = "Can not copy file " + fileName + " from " + fromDirName + " to " + toDirName;
            Util_Logger.error(logger, message, e);
        }

        return result;

    }

    /**
     * This method is used to scan a directory and return all files, which meet
     * the filename filter requirement.
     *
     * @param dirName String
     * @param filter FilenameFilter
     * @return String[] fileNameList
     */
    public String[] scanDir(String dirName, FilenameFilter filter) {

        try {

            String message;
            File dir = new File(dirName);

            if (!dir.exists()) {
               dir.mkdirs();
            }
            // First, we have to test if the directory is a valid directory.
            if (!dir.exists()) {
                message = "Can not list files, " + dirName + " does not exist";
                Util_Logger.warning(logger, message);
                String[] s = null;
                return s;
            }

            if (!dir.isDirectory()) {
                message = "Can not list files, " + dirName + " is not a directory";
                Util_Logger.warning(logger, message);
                String[] s = null;
                return s;
            }

            if (filter != null) {
                return dir.list(filter);
            } else {
                return dir.list();
            }

        } catch (Exception e) {
            Util_Logger.error(logger, "Exception occured when listing file in " + dirName, e);
            String[] s = null;
            return s;
        }

    }


    /**
     * This method is used to select specified files from the FTP returned list
     *
     * @param rawFileList String[]
     * @param fileTypes Vector
     * @param fileExtension String
     * @return Object[] fileNameList
     */
    public Object[] scanFTPFileList(String[] rawFileList, Vector<String> fileTypes, String fileExtension) {
        ArrayList<String> fileList = new ArrayList<String>();

        for (int i = 0; i < rawFileList.length; i++) {
            if (rawFileList[i].endsWith(fileExtension) && rawFileList[i].length() > 0) {

                int index = rawFileList[i].lastIndexOf("/");
                String fileName = rawFileList[i].substring(index + 1);

                // File extension correct
                if ((fileTypes != null) && (fileTypes.size() > 0)) {
                    // there are fileTypes specified
                    for (int j = 0; j < fileTypes.size(); j++) {
                        String fileType = fileTypes.elementAt(j);
                        try {
                            Pattern pattern = Pattern.compile(fileType);
                            Matcher matcher = pattern.matcher(fileName);
                            if (matcher.find()) {
                                fileList.add(fileName);
                                break;
                            }
                        } catch (Exception e) {
                            Util_Logger.error(logger, "Regular expression failure in scanFTPFileList", e);
                        }
                    }

                } else {
                    // There is no fileTypes specified, we have to process all files
                    fileList.add(fileName);
                }
            }
        }

        return fileList.toArray();
    }

    /**
     * This method is used to generate file name with new extension
     *
     * @param oldFileName String
     * @param newFileNameExt (without the .) String
     * @return newFileName String
     */
    public String replaceFileName(String oldFileName, String newFileNameExt) {

        // Assuming that the file names are identicial.
        // Only the extensions are different (e.g. .ack ~ .zip)

        String newFileName = "";
        int index;

        if ((oldFileName != null) && (oldFileName.length() > 0)) {
            index = oldFileName.lastIndexOf(".");
            if (index >= 0) {
                newFileName = oldFileName.substring(0, index + 1) + newFileNameExt;
            }
        }

        return newFileName;
    }

    // This method is used to copy file from one directory to another.
    // It includes all the I/O processes.
    public void copyFile(File sourceFile, File destFile) throws IOException {

        if (!destFile.getParentFile().exists()) {
            destFile.getParentFile().mkdirs();
        }
        if (Constants.getConstant().getFileOverwrite() && destFile.exists()) {
            if (destFile.delete()) {
                Util_Logger.info(logger, "Overwriting existing file - " + destFile.getAbsolutePath());
            } else {
                Util_Logger.warning(logger, "Can not overwrite file - " + destFile.getAbsolutePath());
            }
        }

        byte[] buffer = new byte[8 * 1024];
        int count = 0;
        try (FileInputStream  in  = new FileInputStream(sourceFile);
             FileOutputStream out = new FileOutputStream(destFile);) {
           do {
              out.write(buffer, 0, count);
              count = in.read(buffer, 0, buffer.length);
           } while (count != -1);
        }
    }

    public boolean fileNameCompare(String fileName1, String fileName2) {
        String workName1 = fileName1.substring(0, fileName1.lastIndexOf("."));
        String workName2 = fileName2.substring(0, fileName2.lastIndexOf("."));
        if (workName1.compareTo(workName2) == 0) {
            return true;
        } else {
            return false;
        }
    }
}

