/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 * 28-Aug-2012     Oleg Pevzner       7984 Java upgrade from 6 to 7
 */

package com.nemmco.participantbatcher.process;

import com.nemmco.participantbatcher.ftp.ComProcess;
import com.nemmco.participantbatcher.util.Constants;
import com.nemmco.participantbatcher.util.HttpDataStore;
import com.nemmco.participantbatcher.util.Util_Logger;
import com.nemmco.participantbatcher.xml.AckFileWriter;
import com.nemmco.participantbatcher.xml.ZipFileInfo;
import com.nemmco.participantbatcher.xml.ZipFileValidator;

import java.io.File;

import org.apache.log4j.Logger;

/**
 * This class is used to control the process of User Receives Zip files via FTP
 *
 * @author Kevin Jin
 * version 1.0 - 28 Aug 2003 - Creation
 * version 1.1 - 4 Nov 2003 modified logging
 * version 1.3 - 19-May-2004 added stop file processing, wait only if file downloaded, event logging, log4j
 * version 1.3.1 - 16-Nov-2004 fixed -ve ack file if cant uncompress zip file, low/high water mark counting and logic
 * version 1.4.0 - 28-Feb-2005 Added share as alternative to FTP
 * version 1.4.2 - 26-Jan-2006 Added HttpDataStore
 * version 1.4.4 - 30-Apr-2006 Added a recieve SET FILE TYPE to fix an unzip problem after disconnection using FTP EDT
 * version 1.5.0 - 23-June-2006 Changed stopping for Instance Batcher functionality
 * version 1.5.1 - 1-Nov-2006 Made BatcherFileFilter generic to handle a file type or a set of file types, replaces AckFileFilter
 * version 1.5.4 - 8-Aug-2007 InstanceBatcher2 return work done
 * version 2.0.1 - 15-Nov-2010 Peer review work
 * @version 2.0.2 - 19-Nov-2010 Peer review work
 * @author Deepak Gupta
 * @version 46.93 - 6-Mar-2018 - DFCT0021839
 * 				    15-Mar-2019  DFCT0022209 To modify the count of the processed files after filtering
 */

public class ReceiveProcess {

   // Directories, for easy coding
   private String pnInbox = "";
   private String pnOutbox = "";
   private String plOutbox = "";
   private int acksDeleted = 0;
   private int zipsGot = 0;
   private static Logger logger = Logger.getLogger(ReceiveProcess.class);
   private HttpDataStore hds;
   private FileManager fm = null;
   private ComProcess processer = null;
   private ZipFileValidator zfv = null;
   private AckFileWriter afw = null;

   public ReceiveProcess() {
      this.pnInbox = Constants.getConstant().getPNInboxDir();
      this.pnOutbox = Constants.getConstant().getPNOutboxDir();
      this.plOutbox = Constants.getConstant().getPLOutboxDir();
      this.hds = HttpDataStore.getHttpDataStore();
   }

   /**
    * This method is used to control the receive zip files process.
    *
    * @param paramProcesser ComProcess the main communication processereg FTP
    * @param pniRawFileList String[] file list in the participant NEMMCO inbox
    * @param pnoRawFileList String[] file list in the participant NEMMCO outbox
    */
   public int process(ComProcess paramProcesser, String[] pniRawFileList, String[] pnoRawFileList) {

      processer = paramProcesser;

      Util_Logger.debug(logger, "Start Receiving Process");
      long startTime = System.currentTimeMillis();

      fm = new FileManager();
      zfv = new ZipFileValidator();
      afw = new AckFileWriter();

      try {

         zipsGot = 0;
         acksDeleted = 0;
         processScan(pniRawFileList, pnoRawFileList);

         // log an informative message if work was done
         long timeTaken = System.currentTimeMillis() - startTime;
         if (acksDeleted + zipsGot > 0) {
            Util_Logger.info(logger, "End Recv. " + acksDeleted + " acks deleted, " + zipsGot + " zips got, " + timeTaken + " msec");
         } else {
            Util_Logger.debug(logger, "End Recv. " + acksDeleted + " acks deleted, " + zipsGot + " zips got, " + timeTaken + " msec");
         }
      } finally {
         hds.setZipsRecieved(zipsGot);
         hds.setAcksSent(acksDeleted);
      }
      return zipsGot + acksDeleted;
   }

   public void processScan(String[] pniRawFileList, String[] pnoRawFileList) {

      // Get the .ZIP file list from the PNOutbox
      Object[] pnoZIPFileList = fm.scanFTPFileList(pnoRawFileList, Constants.getConstant().getFileTypes(), ".zip");

      // Get the .ack file list from the PNInbox
      Object[] pniACKFileList = fm.scanFTPFileList(pniRawFileList, Constants.getConstant().getFileTypes(), ".ack");

      // For each file in the ACK file list, if there is no matching ZIP file
      // in the ZIP file list, delete the ACK file via FTP
      // If the delete action failed, there will be no state change
      if ((pniACKFileList != null) && (pniACKFileList.length > 0)) {

         Util_Logger.debug(logger, pniACKFileList.length + " ACK files will be processed");

         for (int c1 = 0; c1 < pniACKFileList.length; c1++) {
            String pniACKFile = (String)pniACKFileList[c1];
            String pnoZIPFile = fm.replaceFileName(pniACKFile, "zip");
            boolean hasMatch = false;

            if ((pnoZIPFileList != null) && (pnoZIPFileList.length > 0)) {
               for (int c2 = 0; c2 < pnoZIPFileList.length; c2++) {
                  if (((String)pnoZIPFileList[c2]).indexOf(pnoZIPFile) >= 0) {
                     hasMatch = true;
                     break;
                  }
               }
            }

            // delete the ack file if there is no match
            if (!hasMatch) {
               processer.deleteFile(pnInbox + "/" + pniACKFile);
               Util_Logger.debug(logger, "Delete file " + pnInbox + "/" + pniACKFile + " via FTP succeed");
               acksDeleted++;
            }

         }

         Util_Logger.debug(logger, pniACKFileList.length + " ACK files have been processed");
      }

      //String[] ploFileList = fm.scanDir(plOutbox, null);
      String[] ploFileList = fm.scanDir(plOutbox, new BatcherFileFilter(new String[] { ".zip", ".nack", ".got" }));

      // Test how many files can be downloaded based on watermarks
      int downloadCount = 0;
      if (ploFileList != null) {
         if (ploFileList.length <= Constants.getConstant().getOutboxLowMark()) {
            // the reason to add 2 - below and minus 2 - above is because there
            // 2 sub-directories (ZIP, NACK) in the participant local inbox directory
            downloadCount = (Constants.getConstant().getOutboxHighMark()) - ploFileList.length;
         }
      }

      // Start to download ZIP files
      if (downloadCount > 0) {

         int realDownloadCount = downloadCount;
         if (downloadCount > pnoZIPFileList.length) {
            realDownloadCount = pnoZIPFileList.length;
         }

         Util_Logger.debug(logger, realDownloadCount + " ZIP files will be downloaded");

         // Get the .got file list from plOutbox
         String[] ploGOTFileList = fm.scanDir(plOutbox, new BatcherFileFilter(".got"));

         for (int c3 = 0; c3 < pnoZIPFileList.length; c3++) {
            // First test if there is matching .ack or .got file
            boolean hasMatch = false;
            String pnoZIPFile = (String)pnoZIPFileList[c3];

            if ((ploGOTFileList != null) && (ploGOTFileList.length > 0)) {
               String ploGOTFile = fm.replaceFileName(pnoZIPFile, "got");
               for (int c4 = 0; c4 < ploGOTFileList.length; c4++) {
                  if (ploGOTFileList[c4].indexOf(ploGOTFile) >= 0) {
                     hasMatch = true;
                     break;
                  }
               }
            }

            if (!hasMatch) {
               if ((pniACKFileList != null) && (pniACKFileList.length > 0)) {
                  String pniACKFile = fm.replaceFileName(pnoZIPFile, "ack");
                  for (int c5 = 0; c5 < pniACKFileList.length; c5++) {
                     if (((String)pniACKFileList[c5]).indexOf(pniACKFile) >= 0) {
                        hasMatch = true;
                        break;
                     }
                  }
               }
            }

            if (!hasMatch) {
               // Download the ZIP file
               boolean success = false;

               String ploGettingFile = plOutbox + "/" + fm.replaceFileName(pnoZIPFile, "getting");
               if (processer.setFileType(Constants.BINARY_FILE_TYPE)) { //v1.4.4
                  if (processer.retrieveFile(pnOutbox + "/" + pnoZIPFile, new File(ploGettingFile))) {
                     // download successfully // fails
                     if (fm.renameFile(plOutbox, fm.replaceFileName(pnoZIPFile, "getting"), fm.replaceFileName(pnoZIPFile, "got"))) {
                        success = true;
                        Util_Logger.debug(logger, "Download file " + ploGettingFile + " succeed");
                        zipsGot++;
                     }
                  }
               }

               if (success) {
                  downloadCount = downloadCount - 1;
                  if (!(downloadCount > 0)) {
                     break;
                  }
               } else {
                  fm.deleteFile(ploGettingFile); //fails
               }

               Util_Logger.debug(logger, "Waiting for " + Constants.getConstant().getFileWaitTime());

               //Thread t = new Thread(); // java 1.6
               try {
                  Thread.sleep(Constants.getConstant().getFileWaitTime());
               } catch (Exception e) {
                  ; // do nothing
               }

            }

         }

         Util_Logger.debug(logger, realDownloadCount + " ZIP files have been downloaded");

      }

      // Start to validate got file and generate ack response

      // Re-get the .got file list from plOutbox
      String[] ploGOTFileList = fm.scanDir(plOutbox, new BatcherFileFilter(".got"));
      if (ploGOTFileList != null) {
         hds.setPloZipFileCount(ploGOTFileList.length);
      }

      if ((ploGOTFileList != null) && (ploGOTFileList.length > 0)) {
         Util_Logger.debug(logger, ploGOTFileList.length + " ACK files will be generated and processed");

         for (int c6 = 0; c6 < ploGOTFileList.length; c6++) {
            String ploGOTFile = ploGOTFileList[c6];
            ZipFileInfo zfi = zfv.validate(plOutbox, ploGOTFile);

            if (zfi.isProcessSuccess()) {
               // the process was successful

					if (zfi.isFileEligibleToBeProcessed()) {
						// the file is eligible to be processed as per "acceptTxTypes, rejectTxTypes, acceptSchemas, rejectSchemas" properties
						// generate the ack file and upload via ftp
						String ploACKFile = fm.replaceFileName(ploGOTFile, "ack");
						String ploTMPFile = fm.replaceFileName(ploGOTFile, "tmp");
						afw.generateAck(plOutbox + "/" + ploACKFile, zfi);
						boolean succeedUpload = false;
						if (processer.setFileType(Constants.ASCII_FILE_TYPE)) { // v1.4.4
							if (processer.storeFile(pnInbox + "/" + ploTMPFile, new File(plOutbox + "/" + ploACKFile))) {
								if (processer.rename(pnInbox + "/" + ploTMPFile, pnInbox + "/" + ploACKFile)) {
									succeedUpload = true;
								}
							}
						}
						if (!succeedUpload) {
							fm.deleteFile(plOutbox + "/" + ploACKFile);
						} else {
							// upload the ACK file success
							Util_Logger.debug(logger, "Upload " + plOutbox + "/" + ploACKFile + " successfully");

							// delete the ack file
							fm.deleteFile(plOutbox + "/" + ploACKFile);

							if (zfi.isValid()) {
								fm.renameFile(plOutbox, ploGOTFile, fm.replaceFileName(ploGOTFile, "zip"));
							} else {
								fm.renameFile(plOutbox, ploGOTFile, fm.replaceFileName(ploGOTFile, "nack"));
							}
						}
					} else {
						// if the file is not eligible to be processed, then delete the GOT file.
					   fm.deleteFile(plOutbox + "/" + ploGOTFile);
					   zipsGot--;
					}
            } else {

               // the unzipping did not work

               // let's generate the ack file and upload via ftp
               String ploACKFile = fm.replaceFileName(ploGOTFile, "ack");
               String ploTMPFile = fm.replaceFileName(ploGOTFile, "tmp");
               afw.generateZipNack(plOutbox + "/" + ploACKFile);
               boolean succeedUpload = false;
               if (processer.setFileType(Constants.ASCII_FILE_TYPE)) { // v1.4.4
                  if (processer.storeFile(pnInbox + "/" + ploTMPFile, new File(plOutbox + "/" + ploACKFile))) {
                     if (processer.rename(pnInbox + "/" + ploTMPFile, pnInbox + "/" + ploACKFile)) {
                        succeedUpload = true;
                     }
                  }
               }
               if (!succeedUpload) {
                  fm.deleteFile(plOutbox + "/" + ploACKFile);
               } else {
                  // upload the ACK file success
                  Util_Logger.debug(logger, "Upload of -ve ack " + plOutbox + "/" + ploACKFile + " successfully");

                  // delete the ack file
                  fm.deleteFile(plOutbox + "/" + ploACKFile);
                  fm.renameFile(plOutbox, ploGOTFile, fm.replaceFileName(ploGOTFile, "nack"));

               }

            }

            zfi = null; // v1.4.4 control memory leak with large numbers of invalid zip files
            Util_Logger.debug(logger, "Finished process " + plOutbox + "/" + ploGOTFile);

         }

         Util_Logger.debug(logger, ploGOTFileList.length + " ACK files have been generated and processed");

      }

   }

}
